// ------------------------------------------------------------------------
// 	Estimating Katayama-Kim version of the Two-Sector Model		
//  Only 4-quarter-ahead news shocks
// ------------------------------------------------------------------------

close all

// ------------------------------------------------------------------------
// Declare endogenous variables

var C, 		
	N,		
	Nc, 	
	Ni, 	
	lambda,
	wc, 	
	wi, 	
	rc, 	
	ri, 	
	tauc, 	
	taui, 	
	p, 		
	I, 		
	Ic, 	
	Ii, 	
	uc, 	
	ui, 	
	Kc, 	
	Ki, 	
	A, 		
	Z,
	G,		
	zi, 	
	b,
	muc
	mui,		
	va4_1,
	va4_2,
	va4_3,
	va4_4,
	vz4_1,
	vz4_2,
	vz4_3,
	vz4_4,
	vzi4_1,
	vzi4_2,
	vzi4_3,
	vzi4_4,
	vmuc4_1,
	vmuc4_2,
	vmuc4_3,
	vmuc4_4,
	vmui4_1,
	vmui4_2,
	vmui4_3,
	vmui4_4,
    dc,
    di,
    dw,
	dhc,
	dhi,
	dp;

// ------------------------------------------------------------------------
// Declare exogenous variable
varexo 	va0,
		va4,
		vz0,
		vz4,
  		vzi0,
		vzi4,
		vb0, 
		vmuc0,
		vmuc4,
		vmui0,
		vmui4;
				
// ------------------------------------------------------------------------
// Declare parameters
parameters 	alpha,  
			beta,
			h,   
			kappaC, 
			kappaI, 
			sigma,	
			eta_,	
			theta_, 
			gz2,	
			deltaC, 
			phi_c,   
			phi_i,
			rhoA,   
			rhoZ,	
			rhoZi   
			rhob,
			rhomuc,
			rhomui,
			sc,
			muc_ss;		

// --- Assigning Parameter Values
alpha = 0.36;
beta = 0.985;
deltaC = 0.025;
sc = 0.78;

muc_ss = 1.15;

// ------------------------------------------------------------------------
// Model Specification
model(linear);

// --- Auxiliary Parameters

# sigma_ = 1/sigma;

# deltaI = deltaC;
# gz = gz2/400;
# gc = exp( gz/(1-alpha) );
# beta_ = gc^(-sigma_)*h*beta;
# h_ = h/gc; 
# chiC = gc^(sigma_)/beta  + deltaC - 1;
# chiI = gc^(sigma_)/beta  + deltaI - 1;
# omegaN = (1 - beta_)/(1 - h_)*(1 - sigma)*(1 - alpha)/(sc*muc_ss);
# deltaC2 = gc - (1 - deltaC);
# deltaI2 = gc - (1 - deltaI);
# IcI = ( 1 + deltaI2*chiC*(1 - sc)/( deltaC2*chiI*sc ) )^(-1);
# theta = 1/theta_;

//	1	Aux variable
G = 1/(1-alpha)*Z;

//   2  Consumption Demand
sigma*(1 - beta_)*lambda = ( h_*C(-1) - (1 + beta_*h_)*C + beta_*C(+1) - h_*G + beta_*G(+1) )/(1 - h_) + omegaN*( N - beta_*N(+1) ) + sigma*( b - beta_*b(+1) );

//   2  Labor Supply to Consumption Sector
lambda + wc = (1 - sigma_)/(1 - h_)*( C - h_*C(-1) + h_*G ) + ( (sigma_ - 1)*omegaN - 1/theta + eta_ )*N + 1/theta*Nc + b + muc;

//   3  Labor Supply to Investment Sector
lambda + wi = (1 - sigma_)/(1 - h_)*( C - h_*C(-1) + h_*G ) + ( (sigma_ - 1)*omegaN - 1/theta + eta_ )*N + 1/theta*Ni + b + mui;

//   4  Capital Service Supply to Consumption Sector
lambda + rc = tauc + kappaC*uc;

//   5  Capital Service Supply to Investment Sector
lambda + ri = taui + kappaI*ui;

//   6  Capital Euler Equation in Consumption Sector
tauc + sigma_*G(+1) = beta*gc^(-sigma_)*( chiC*lambda(+1) + chiC*rc(+1) + (1 - deltaC)*tauc(+1) );

//   7  Capital Euler Equation in Investment Sector
taui + sigma_*G(+1) = beta*gc^(-sigma_)*( chiI*lambda(+1) + chiI*ri(+1) + (1 - deltaI)*taui(+1) );

//  8  Investment in Consumption Sector
lambda + p = tauc - phi_c*gc^(2)*( Ic - Ic(-1) + G ) 
			 + beta*gc^(3-sigma_)*phi_c*( Ic(+1) - Ic + G(+1) );

//  8  Investment in Investment Sector
lambda + p = taui - phi_i*gc^(2)*( Ii - Ii(-1) + G ) 
			 + beta*gc^(3-sigma_)*phi_i*( Ii(+1) - Ii + G(+1) );
			 
//  10  Labor Demand in Consumption Sector
wc = C - Nc;

//  11  Labor Demand in Investment Sector
wi = I - Ni + p;

//  12  Capital Service Demand in Consumption Sector
rc = C - uc - Kc(-1) + G;

//  13  Capital Service Demand in Investment Sector
ri = I - ui - Ki(-1) + p + G;

//  14  Production Function in Consumption Sector
C = -alpha*G + A + alpha*uc + alpha*Kc(-1) + (1 - alpha)*Nc;

//  15  Production Function in Investment Sector
I = -alpha*G + A + zi + alpha*ui + alpha*Ki(-1) + (1 - alpha)*Ni;

//  16  Aggregate Labor Index
N = sc*Nc + (1 - sc)*Ni;

//  17  Aggregate Investment
I = IcI*Ic + (1 - IcI)*Ii;

//  18  Capital Accumulation in Consumption Sector
gc*Kc = (gc - (1 - deltaC))*Ic - chiC*uc + (1 - deltaC)*( Kc(-1) - G );

//  19  Capital Accumulation in Investment Sector
gc*Ki = (gc - (1 - deltaI))*Ii - chiI*ui + (1 - deltaI)*( Ki(-1) - G );

//  20  Stationary Aggregate TFP Process
A = rhoA*A(-1) + va0 + va4_4;

//  21  Non-Stationary Aggregate TFP Process
Z = rhoZ*Z(-1) + vz0 + vz4_4;

//  22  Investment-sector TFP Process
zi = rhoZi*zi(-1) + vzi0 + vzi4_4;

//  23 Discount factor shock
b = rhob*b(-1) + vb0;

//	24	Consumption-sector Wage Markup Shock
muc = rhomuc*muc(-1) + vmuc0 + vmuc4_4;

//	25	Consumption-sector Wage Markup Shock
mui = rhomui*mui(-1) + vmui0 + vmui4_4;



// --- Information Structure for the Shocks 
// - for stationary aggregate shocks
va4_1 = va4(-1); 	
va4_2 = va4_1(-1);
va4_3 = va4_2(-1);
va4_4 = va4_3(-1);
// - for non-stationary aggregate shocks
vz4_1 = vz4(-1); 	
vz4_2 = vz4_1(-1);
vz4_3 = vz4_2(-1);
vz4_4 = vz4_3(-1);
// -- for investment-specific shocks
vzi4_1 = vzi4(-1); 	
vzi4_2 = vzi4_1(-1);
vzi4_3 = vzi4_2(-1);
vzi4_4 = vzi4_3(-1);
// -- for non-durable wage markup shocks
vmuc4_1 = vmuc4(-1);     	
vmuc4_2 = vmuc4_1(-1);
vmuc4_3 = vmuc4_2(-1);
vmuc4_4 = vmuc4_3(-1);
// -- for durable wage markup shocks
vmui4_1 = vmui4(-1);     	
vmui4_2 = vmui4_1(-1);
vmui4_3 = vmui4_2(-1);
vmui4_4 = vmui4_3(-1);

// --- Observation Equations
dc = C - C(-1);
di = I - I(-1);
dw = sc*(wc - wc(-1)) + (1-sc)*(wi - wi(-1));
dhc = Nc - Nc(-1);
dhi = Ni - Ni(-1);
dp = p - p(-1);

end;

// ------------------------------------------------------------------------
// Description of the steady-state values
steady_state_model;
	C 	    = 0;
	N 	    = 0;
	Nc 	    = 0;
	Ni 	    = 0;
	lambda  = 0;
	wc 	    = 0;
	wi 	    = 0;
	rc 	    = 0;
	ri 	    = 0;
	tauc    = 0;
	taui    = 0;
	p 	    = 0;
	I 	    = 0;
	Ic 	    = 0;
	Ii 	    = 0;
	uc 	    = 0;
	ui 	    = 0;
	Kc 	    = 0;
	Ki 	    = 0;
	A 	    = 0;
	Z		= 0;
	G		= 0;
	zi 	    = 0;
	va4_1	= 0;
	va4_2	= 0;
	va4_3	= 0;
	va4_4	= 0;
	vz4_1	= 0;
	vz4_2	= 0;
	vz4_3	= 0;
	vz4_4	= 0;
	vzi4_1	= 0;
	vzi4_2	= 0;
	vzi4_3	= 0;
	vzi4_4	= 0;
	vmuc4_1	= 0;
	vmuc4_2	= 0;
	vmuc4_3	= 0;
	vmuc4_4	= 0;
	vmui4_1	= 0;
	vmui4_2	= 0;
	vmui4_3	= 0;
	vmui4_4	= 0;
	b		= 0;
	muc	   	= 0;
	mui	   	= 0;
    dc	    = 0;
    di	    = 0;
    dw	    = 0;
    dhc	    = 0;
    dhi	    = 0;
	dp 		= 0;
end;


// ------------------------------------------------------------------------
// Solving for the deterministic steady state
//steady;

// ------------------------------------------------------------------------
// Declaring the observables
varobs dc, di, dw, dhc, dhi, dp; 

// ------------------------------------------------------------------------
// Defining parameters to be estimated
estimated_params;
	kappaC,			inv_gamma_pdf,  1,	  1;
	kappaI,			inv_gamma_pdf,  1,	  1;
	phi_c,   		gamma_pdf,		4,    1;
	phi_i,   		gamma_pdf,		4,    1;
	gz2,			gamma_pdf,		1.65, 	1;
	rhoA,      		beta_pdf, 		0.7,  0.2;
	rhoZ,			beta_pdf,		0.2,  0.2;
	rhoZi,     		beta_pdf, 		0.5,  0.2;
	rhob,      		beta_pdf, 		0.7,  0.2;
	rhomuc,			beta_pdf,		0.5,  0.2;
	rhomui,			beta_pdf,		0.5,  0.2;	
	h,				beta_pdf,		0.5,  0.2;
	eta_,			gamma_pdf,		1,    0.25;
	theta_,			gamma_pdf,		1,    0.5;
	sigma,			beta_pdf, 		0.5, 	0.2;
	stderr vb0, 	inv_gamma_pdf,	0.0173,	0.0173;
	stderr va0,		gamma_pdf, 		0.015,	0.015;   
	stderr va4, 	gamma_pdf, 		0.0043,	0.0043;
	stderr vz0,		gamma_pdf, 		0.015,	0.015;   
	stderr vz4, 	gamma_pdf, 		0.0043,	0.0043;
	stderr vzi0,	gamma_pdf, 		0.015,	0.015;
	stderr vzi4,	gamma_pdf, 		0.0043,	0.0043;
	stderr vmuc0,	gamma_pdf, 		0.015,	0.015;
	stderr vmuc4,	gamma_pdf, 		0.0043,	0.0043;
	stderr vmui0,	gamma_pdf, 		0.015,	0.015;
	stderr vmui4,	gamma_pdf, 		0.0043,	0.0043;
end;

// Setting initial guesses
estimated_params_init;
	 kappaC,		5.2233;
	 kappaI,		0.7356;
	 phi_c,		0.6297;
	 phi_i,		3.6803;
	 gz2,		0.6662;
	 rhoA,		0.9379;
	 rhoZ,		0.5783;
	 rhoZi,		0.8803;
	 rhob,		0.9188;
	 rhomuc,		0.9859;
	 rhomui,		0.9352;
	 h,		0.3286;
	 eta_,		0.8186;
	 theta_,		3.0148;
	 sigma,		0.6637;
	 stderr vb0,		0.0537;
	 stderr va0,		0.0034;
	 stderr va4,		0.0007;
	 stderr vz0,		0.0027;
	 stderr vz4,		0.0009;
	 stderr vzi0,		0.0203;
	 stderr vzi4,		0.0043;
	 stderr vmuc0,		0.0024;
	 stderr vmuc4,		0.0066;
	 stderr vmui0,		0.0050;
	 stderr vmui4,		0.0471;
end;

// Specifying the Bayesian estimation
estimation(
order = 1,
datafile = SectoralData2016,
prefilter = 1,
mh_replic = 300000,
mh_drop = 0.1,
mh_jscale = 0.25,
nograph,
plot_priors = 0,
mode_compute = 6,
first_obs = 1,
mh_nblocks = 1
) C, I, N, Nc, Ni, Ic, Ii, uc, ui, rc, ri, wc, wi, lambda, tauc, taui, dc, di, dw, dhc, dhi, dp;
